@@ -1,4 +1,4 @@
-/* $Id: thunar-uca-editor.c 26421 2007-12-02 13:46:28Z benny $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2005-2007 Benedikt Meurer <benny@xfce.org>
  *
@@ -35,9 +35,6 @@
 
 
 
-static void           thunar_uca_editor_class_init      (ThunarUcaEditorClass   *klass);
-static void           thunar_uca_editor_init            (ThunarUcaEditor        *uca_editor);
-static void           thunar_uca_editor_finalize        (GObject                *object);
 static const gchar   *thunar_uca_editor_get_icon_name   (const ThunarUcaEditor  *uca_editor);
 static void           thunar_uca_editor_set_icon_name   (ThunarUcaEditor        *uca_editor,
                                                          const gchar            *icon_name);
@@ -58,12 +55,11 @@ struct _ThunarUcaEditor
 {
   GtkDialog __parent__;
 
-  GtkTooltips *tooltips;
-
   GtkWidget   *name_entry;
   GtkWidget   *description_entry;
   GtkWidget   *icon_button;
   GtkWidget   *command_entry;
+  GtkWidget   *sn_button;
   GtkWidget   *parameter_entry;
   GtkWidget   *patterns_entry;
   GtkWidget   *directories_button;
@@ -83,10 +79,6 @@ THUNARX_DEFINE_TYPE (ThunarUcaEditor, thunar_uca_editor, GTK_TYPE_DIALOG);
 static void
 thunar_uca_editor_class_init (ThunarUcaEditorClass *klass)
 {
-  GObjectClass *gobject_class;
-
-  gobject_class = G_OBJECT_CLASS (klass);
-  gobject_class->finalize = thunar_uca_editor_finalize;
 }
 
 
@@ -119,11 +111,6 @@ thunar_uca_editor_init (ThunarUcaEditor *uca_editor)
   gtk_window_set_destroy_with_parent (GTK_WINDOW (uca_editor), TRUE);
   gtk_window_set_resizable (GTK_WINDOW (uca_editor), FALSE);
 
-  /* allocate shared tooltips object */
-  uca_editor->tooltips = gtk_tooltips_new ();
-  g_object_ref (G_OBJECT (uca_editor->tooltips));
-  gtk_object_sink (GTK_OBJECT (uca_editor->tooltips));
-
   notebook = gtk_notebook_new ();
   gtk_container_set_border_width (GTK_CONTAINER (notebook), 6);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (uca_editor)->vbox), notebook, TRUE, TRUE, 0);
@@ -133,7 +120,7 @@ thunar_uca_editor_init (ThunarUcaEditor *uca_editor)
      Basic
    */
   label = gtk_label_new (_("Basic"));
-  table = gtk_table_new (6, 2, FALSE);
+  table = gtk_table_new (7, 2, FALSE);
   gtk_table_set_col_spacings (GTK_TABLE (table), 12);
   gtk_table_set_row_spacings (GTK_TABLE (table), 6);
   gtk_container_set_border_width (GTK_CONTAINER (table), 12);
@@ -146,7 +133,7 @@ thunar_uca_editor_init (ThunarUcaEditor *uca_editor)
   gtk_widget_show (label);
 
   uca_editor->name_entry = g_object_new (GTK_TYPE_ENTRY, "activates-default", TRUE, NULL);
-  gtk_tooltips_set_tip (uca_editor->tooltips, uca_editor->name_entry, _("The name of the action that will be displayed in the context menu."), NULL);
+  gtk_widget_set_tooltip_text (uca_editor->name_entry, _("The name of the action that will be displayed in the context menu."));
   gtk_table_attach (GTK_TABLE (table), uca_editor->name_entry, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), uca_editor->name_entry);
   gtk_widget_grab_focus (uca_editor->name_entry);
@@ -164,8 +151,8 @@ thunar_uca_editor_init (ThunarUcaEditor *uca_editor)
   gtk_widget_show (label);
 
   uca_editor->description_entry = g_object_new (GTK_TYPE_ENTRY, "activates-default", TRUE, NULL);
-  gtk_tooltips_set_tip (uca_editor->tooltips, uca_editor->description_entry, _("The description of the action that will be displayed as tooltip "
-                                                                               "in the statusbar when selecting the item from the context menu."), NULL);
+  gtk_widget_set_tooltip_text (uca_editor->description_entry, _("The description of the action that will be displayed as tooltip "
+                                                                "in the statusbar when selecting the item from the context menu."));
   gtk_table_attach (GTK_TABLE (table), uca_editor->description_entry, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), uca_editor->description_entry);
   gtk_widget_show (uca_editor->description_entry);
@@ -186,18 +173,18 @@ thunar_uca_editor_init (ThunarUcaEditor *uca_editor)
   gtk_widget_show (hbox);
 
   uca_editor->command_entry = g_object_new (GTK_TYPE_ENTRY, "activates-default", TRUE, NULL);
-  gtk_tooltips_set_tip (uca_editor->tooltips, uca_editor->command_entry, _("The command (including the necessary parameters) to perform the action. "
-                                                                           "See the command parameter legend below for a list of supported parameter "
-                                                                           "variables, which will be substituted when launching the command. When "
-                                                                           "upper-case letters (e.g. %F, %D, %N) are used, the action will be applicable "
-                                                                           "even if more than one item is selected. Else the action will only be "
-                                                                           "applicable if exactly one item is selected."), NULL);
+  gtk_widget_set_tooltip_text (uca_editor->command_entry, _("The command (including the necessary parameters) to perform the action. "
+                                                            "See the command parameter legend below for a list of supported parameter "
+                                                            "variables, which will be substituted when launching the command. When "
+                                                            "upper-case letters (e.g. %F, %D, %N) are used, the action will be applicable "
+                                                            "even if more than one item is selected. Else the action will only be "
+                                                            "applicable if exactly one item is selected."));
   gtk_box_pack_start (GTK_BOX (hbox), uca_editor->command_entry, TRUE, TRUE, 0);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), uca_editor->command_entry);
   gtk_widget_show (uca_editor->command_entry);
 
   button = gtk_button_new ();
-  gtk_tooltips_set_tip (uca_editor->tooltips, button, _("Browse the file system to select an application to use for this action."), NULL);
+  gtk_widget_set_tooltip_text (button, _("Browse the file system to select an application to use for this action."));
   gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
   g_signal_connect_swapped (G_OBJECT (button), "clicked", G_CALLBACK (thunar_uca_editor_command_clicked), uca_editor);
   gtk_widget_show (button);
@@ -213,17 +200,24 @@ thunar_uca_editor_init (ThunarUcaEditor *uca_editor)
   atk_relation_set_add (relations, relation);
   g_object_unref (G_OBJECT (relation));
 
+  uca_editor->sn_button = gtk_check_button_new_with_label (_("Use Startup Notification"));
+  gtk_widget_set_tooltip_text (uca_editor->sn_button, _("Enable this option if you want a waiting cursor to be shown while the "
+                                                        "action is launched. This is also highly recommended if you have focus "
+                                                        "stealing prevention enabled in your window manager."));
+  gtk_table_attach (GTK_TABLE (table), uca_editor->sn_button, 1, 2, 3, 4, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+  gtk_widget_show (uca_editor->sn_button);
+
   label = g_object_new (GTK_TYPE_LABEL, "label", _("_Icon:"), "use-underline", TRUE, "xalign", 0.0f, NULL);
-  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4, GTK_FILL, GTK_FILL, 0, 0);
+  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 4, 5, GTK_FILL, GTK_FILL, 0, 0);
   gtk_widget_show (label);
 
   align = gtk_alignment_new (0.0f, 0.5f, 0.0f, 0.0f);
-  gtk_table_attach (GTK_TABLE (table), align, 1, 2, 3, 4, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+  gtk_table_attach (GTK_TABLE (table), align, 1, 2, 4, 5, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
   gtk_widget_show (align);
 
   uca_editor->icon_button = gtk_button_new_with_label (_("No icon"));
-  gtk_tooltips_set_tip (uca_editor->tooltips, uca_editor->icon_button, _("Click this button to select an icon file that will be displayed "
-                                                                         "in the context menu in addition to the action name chosen above."), NULL);
+  gtk_widget_set_tooltip_text (uca_editor->icon_button, _("Click this button to select an icon file that will be displayed "
+                                                          "in the context menu in addition to the action name chosen above."));
   gtk_container_add (GTK_CONTAINER (align), uca_editor->icon_button);
   g_signal_connect_swapped (G_OBJECT (uca_editor->icon_button), "clicked", G_CALLBACK (thunar_uca_editor_icon_clicked), uca_editor);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), uca_editor->icon_button);
@@ -237,11 +231,11 @@ thunar_uca_editor_init (ThunarUcaEditor *uca_editor)
   g_object_unref (G_OBJECT (relation));
 
   align = g_object_new (GTK_TYPE_ALIGNMENT, "height-request", 12, NULL);
-  gtk_table_attach (GTK_TABLE (table), align, 0, 2, 4, 5, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+  gtk_table_attach (GTK_TABLE (table), align, 0, 2, 5, 6, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
   gtk_widget_show (align);
 
   hbox = gtk_hbox_new (FALSE, 6);
-  gtk_table_attach (GTK_TABLE (table), hbox, 0, 2, 5, 6, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+  gtk_table_attach (GTK_TABLE (table), hbox, 0, 2, 6, 7, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
   gtk_widget_show (hbox);
 
   image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_INFO, GTK_ICON_SIZE_DND);
@@ -378,10 +372,10 @@ thunar_uca_editor_init (ThunarUcaEditor *uca_editor)
   gtk_widget_show (label);
 
   uca_editor->patterns_entry = g_object_new (GTK_TYPE_ENTRY, "activates-default", TRUE, "text", "*", NULL);
-  gtk_tooltips_set_tip (uca_editor->tooltips, uca_editor->patterns_entry, _("Enter a list of patterns that will be used to determine "
-                                                                            "whether this action should be displayed for a selected "
-                                                                            "file. If you specify more than one pattern here, the list "
-                                                                            "items must be separated with semicolons (e.g. *.txt;*.doc)."), NULL);
+  gtk_widget_set_tooltip_text (uca_editor->patterns_entry, _("Enter a list of patterns that will be used to determine "
+                                                             "whether this action should be displayed for a selected "
+                                                             "file. If you specify more than one pattern here, the list "
+                                                             "items must be separated with semicolons (e.g. *.txt;*.doc)."));
   gtk_table_attach (GTK_TABLE (table), uca_editor->patterns_entry, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), uca_editor->patterns_entry);
   gtk_widget_show (uca_editor->patterns_entry);
@@ -464,19 +458,6 @@ thunar_uca_editor_init (ThunarUcaEditor *uca_editor)
 
 
 
-static void
-thunar_uca_editor_finalize (GObject *object)
-{
-  ThunarUcaEditor *uca_editor = THUNAR_UCA_EDITOR (object);
-
-  /* release the tooltips */
-  g_object_unref (G_OBJECT (uca_editor->tooltips));
-
-  (*G_OBJECT_CLASS (thunar_uca_editor_parent_class)->finalize) (object);
-}
-
-
-
 static void
 thunar_uca_editor_command_clicked (ThunarUcaEditor *uca_editor)
 {
@@ -790,6 +771,7 @@ thunar_uca_editor_load (ThunarUcaEditor *uca_editor,
   gchar         *command;
   gchar         *icon;
   gchar         *name;
+  gboolean       startup_notify;
 
   g_return_if_fail (THUNAR_UCA_IS_EDITOR (uca_editor));
   g_return_if_fail (THUNAR_UCA_IS_MODEL (uca_model));
@@ -803,6 +785,7 @@ thunar_uca_editor_load (ThunarUcaEditor *uca_editor,
                       THUNAR_UCA_MODEL_COLUMN_TYPES, &types,
                       THUNAR_UCA_MODEL_COLUMN_ICON, &icon,
                       THUNAR_UCA_MODEL_COLUMN_NAME, &name,
+                      THUNAR_UCA_MODEL_COLUMN_STARTUP_NOTIFY, &startup_notify,
                       -1);
 
   /* setup the new selection */
@@ -816,6 +799,7 @@ thunar_uca_editor_load (ThunarUcaEditor *uca_editor,
   gtk_entry_set_text (GTK_ENTRY (uca_editor->patterns_entry), (patterns != NULL) ? patterns : "");
   gtk_entry_set_text (GTK_ENTRY (uca_editor->command_entry), (command != NULL) ? command : "");
   gtk_entry_set_text (GTK_ENTRY (uca_editor->name_entry), (name != NULL) ? name : "");
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (uca_editor->sn_button), startup_notify);
 
   /* cleanup */
   g_free (description);
@@ -850,6 +834,7 @@ thunar_uca_editor_save (ThunarUcaEditor *uca_editor,
                            gtk_entry_get_text (GTK_ENTRY (uca_editor->description_entry)),
                            thunar_uca_editor_get_icon_name (uca_editor),
                            gtk_entry_get_text (GTK_ENTRY (uca_editor->command_entry)),
+                           gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (uca_editor->sn_button)),
                            gtk_entry_get_text (GTK_ENTRY (uca_editor->patterns_entry)),
                            thunar_uca_editor_get_types (uca_editor));
 }
