@@ -1,6 +1,7 @@
-/* $Id: thunar-uca-model.c 26395 2007-11-28 19:15:27Z benny $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
+ * Copyright (c) 2009 Jannis Pohlmann <jannis@xfce.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -44,8 +45,13 @@
 #include <glib/gi18n-lib.h>
 #include <glib/gstdio.h>
 
+#include <gio/gio.h>
+
+#include <gtk/gtk.h>
+
+#include <libxfce4util/libxfce4util.h>
+
 #include <thunar-uca/thunar-uca-model.h>
-#include <thunar-vfs/thunar-vfs.h>
 
 
 
@@ -68,6 +74,7 @@ typedef enum
   PARSER_ICON,
   PARSER_NAME,
   PARSER_COMMAND,
+  PARSER_STARTUP_NOTIFY,
   PARSER_PATTERNS,
   PARSER_DESCRIPTION,
   PARSER_DIRECTORIES,
@@ -80,9 +87,7 @@ typedef enum
 
 
 
-static void               thunar_uca_model_class_init       (ThunarUcaModelClass  *klass);
 static void               thunar_uca_model_tree_model_init  (GtkTreeModelIface    *iface);
-static void               thunar_uca_model_init             (ThunarUcaModel       *uca_model);
 static void               thunar_uca_model_finalize         (GObject              *object);
 static GtkTreeModelFlags  thunar_uca_model_get_flags        (GtkTreeModel         *tree_model);
 static gint               thunar_uca_model_get_n_columns    (GtkTreeModel         *tree_model);
@@ -157,11 +162,11 @@ struct _ThunarUcaModelItem
   gchar         *description;
   gchar         *icon;
   gchar         *command;
+  guint          startup_notify : 1;
   gchar        **patterns;
   ThunarUcaTypes types;
 
   /* derived attributes */
-  gchar         *stock_id;
   gboolean       multiple_selection : 1;
 };
 
@@ -179,6 +184,7 @@ typedef struct
   GString        *command;
   GString        *patterns;
   GString        *description;
+  gboolean        startup_notify;
   gboolean        description_use;
   guint           description_match;
   ThunarUcaTypes  types;
@@ -320,15 +326,15 @@ thunar_uca_model_get_column_type (GtkTreeModel *tree_model,
     case THUNAR_UCA_MODEL_COLUMN_COMMAND:
       return G_TYPE_STRING;
 
+    case THUNAR_UCA_MODEL_COLUMN_STARTUP_NOTIFY:
+      return G_TYPE_BOOLEAN;
+
     case THUNAR_UCA_MODEL_COLUMN_PATTERNS:
       return G_TYPE_STRING;
 
     case THUNAR_UCA_MODEL_COLUMN_TYPES:
       return G_TYPE_UINT;
 
-    case THUNAR_UCA_MODEL_COLUMN_STOCK_ID:
-      return G_TYPE_STRING;
-
     case THUNAR_UCA_MODEL_COLUMN_STOCK_LABEL:
       return G_TYPE_STRING;
 
@@ -363,17 +369,17 @@ thunar_uca_model_get_path (GtkTreeModel *tree_model,
                            GtkTreeIter  *iter)
 {
   ThunarUcaModel *uca_model = THUNAR_UCA_MODEL (tree_model);
-  gint            index;
+  gint            idx;
 
   g_return_val_if_fail (THUNAR_UCA_IS_MODEL (uca_model), NULL);
   g_return_val_if_fail (iter->stamp == uca_model->stamp, NULL);
 
   /* determine the index of the iter */
-  index = g_list_position (uca_model->items, iter->user_data);
-  if (G_UNLIKELY (index < 0))
+  idx = g_list_position (uca_model->items, iter->user_data);
+  if (G_UNLIKELY (idx < 0))
     return NULL;
 
-  return gtk_tree_path_new_from_indices (index, -1);
+  return gtk_tree_path_new_from_indices (idx, -1);
 }
 
 
@@ -386,8 +392,6 @@ thunar_uca_model_get_value (GtkTreeModel *tree_model,
 {
   ThunarUcaModelItem *item = ((GList *) iter->user_data)->data;
   ThunarUcaModel     *uca_model = THUNAR_UCA_MODEL (tree_model);
-  GtkIconSource      *icon_source;
-  GtkIconSet         *icon_set;
   gchar              *str;
 
   g_return_if_fail (THUNAR_UCA_IS_MODEL (uca_model));
@@ -399,7 +403,7 @@ thunar_uca_model_get_value (GtkTreeModel *tree_model,
   switch (column)
     {
     case THUNAR_UCA_MODEL_COLUMN_NAME:
-      g_value_set_static_string (value, item->name);
+      g_value_set_static_string (value, item->name ? item->name : "");
       break;
 
     case THUNAR_UCA_MODEL_COLUMN_DESCRIPTION:
@@ -414,6 +418,10 @@ thunar_uca_model_get_value (GtkTreeModel *tree_model,
       g_value_set_static_string (value, item->command);
       break;
 
+    case THUNAR_UCA_MODEL_COLUMN_STARTUP_NOTIFY:
+      g_value_set_boolean (value, item->startup_notify);
+      break;
+
     case THUNAR_UCA_MODEL_COLUMN_PATTERNS:
       str = g_strjoinv (";", item->patterns);
       g_value_take_string (value, str);
@@ -423,31 +431,6 @@ thunar_uca_model_get_value (GtkTreeModel *tree_model,
       g_value_set_uint (value, item->types);
       break;
 
-    case THUNAR_UCA_MODEL_COLUMN_STOCK_ID:
-      if (item->stock_id == NULL && item->icon != NULL)
-        {
-          /* allocate a new icon set for the item */
-          icon_set = gtk_icon_set_new ();
-          icon_source = gtk_icon_source_new ();
-          if (g_path_is_absolute (item->icon))
-            gtk_icon_source_set_filename (icon_source, item->icon);
-          else
-            gtk_icon_source_set_icon_name (icon_source, item->icon);
-          gtk_icon_set_add_source (icon_set, icon_source);
-          gtk_icon_source_free (icon_source);
-
-          /* generate a unique stock-id for the icon */
-          item->stock_id = g_strdup_printf ("thunar-uca-%p", item);
-
-          /* register the icon set */
-          gtk_icon_factory_add (uca_model->icon_factory, item->stock_id, icon_set);
-
-          /* cleanup */
-          gtk_icon_set_unref (icon_set);
-        }
-      g_value_set_static_string (value, item->stock_id);
-      break;
-
     case THUNAR_UCA_MODEL_COLUMN_STOCK_LABEL:
       str = g_markup_printf_escaped ("<b>%s</b>\n%s", (item->name != NULL) ? item->name : "", (item->description != NULL) ? item->description : "");
       g_value_take_string (value, str);
@@ -576,6 +559,7 @@ thunar_uca_model_load_from_file (ThunarUcaModel *uca_model,
   parser.command = g_string_new (NULL);
   parser.patterns = g_string_new (NULL);
   parser.description = g_string_new (NULL);
+  parser.startup_notify = FALSE;
   xfce_stack_push (parser.stack, PARSER_START);
 
   /* parse the file */
@@ -604,7 +588,6 @@ thunar_uca_model_item_reset (ThunarUcaModelItem *item)
   /* release the previous values... */
   g_strfreev (item->patterns);
   g_free (item->description);
-  g_free (item->stock_id);
   g_free (item->command);
   g_free (item->name);
   g_free (item->icon);
@@ -650,6 +633,7 @@ start_element_handler (GMarkupParseContext *context,
           parser->name_match = XFCE_LOCALE_NO_MATCH;
           parser->description_match = XFCE_LOCALE_NO_MATCH;
           parser->types = 0;
+          parser->startup_notify = FALSE;
           g_string_truncate (parser->icon, 0);
           g_string_truncate (parser->name, 0);
           g_string_truncate (parser->command, 0);
@@ -735,6 +719,11 @@ start_element_handler (GMarkupParseContext *context,
 
           xfce_stack_push (parser->stack, PARSER_DESCRIPTION);
         }
+      else if (strcmp (element_name, "startup-notify") == 0)
+        {
+          parser->startup_notify = TRUE;
+          xfce_stack_push (parser->stack, PARSER_STARTUP_NOTIFY);
+        }
       else if (strcmp (element_name, "directories") == 0)
         {
           parser->types |= THUNAR_UCA_TYPE_DIRECTORIES;
@@ -812,6 +801,7 @@ end_element_handler (GMarkupParseContext *context,
                                    parser->description->str,
                                    parser->icon->str,
                                    parser->command->str,
+                                   parser->startup_notify,
                                    parser->patterns->str,
                                    parser->types);
         }
@@ -844,6 +834,11 @@ end_element_handler (GMarkupParseContext *context,
         goto unknown_element;
       break;
 
+    case PARSER_STARTUP_NOTIFY:
+      if (strcmp (element_name, "startup-notify") != 0)
+        goto unknown_element;
+      break;
+
     case PARSER_DIRECTORIES:
       if (strcmp (element_name, "directories") != 0)
         goto unknown_element;
@@ -1021,17 +1016,16 @@ thunar_uca_model_match (ThunarUcaModel *uca_model,
     ThunarUcaTypes types;
   } ThunarUcaFile;
 
-  ThunarVfsMimeDatabase *mime_database;
-  ThunarUcaModelItem    *item;
-  ThunarUcaFile         *files;
-  ThunarVfsInfo         *info;
-  const gchar           *mime_type;
-  gboolean               matches;
-  GList                 *mime_infos;
-  GList                 *paths = NULL;
-  GList                 *lp, *mp;
-  gint                   n_files;
-  gint                   i, m, n;
+  ThunarUcaModelItem *item;
+  ThunarUcaFile      *files;
+  GFileInfo          *info;
+  GFile              *location;
+  const gchar        *mime_type;
+  gboolean            matches;
+  GList              *paths = NULL;
+  GList              *lp;
+  gint                n_files;
+  gint                i, m, n;
 
   g_return_val_if_fail (THUNAR_UCA_IS_MODEL (uca_model), NULL);
 
@@ -1044,33 +1038,29 @@ thunar_uca_model_match (ThunarUcaModel *uca_model,
   files = g_new (ThunarUcaFile, n_files);
   for (lp = file_infos, n = 0; lp != NULL; lp = lp->next, ++n)
     {
-      info = thunarx_file_info_get_vfs_info (lp->data);
-      if (thunar_vfs_path_get_scheme (info->path) != THUNAR_VFS_PATH_SCHEME_FILE)
+      location = thunarx_file_info_get_location (lp->data);
+
+      if (!g_file_has_uri_scheme (location, "file"))
         {
           /* cannot handle non-local files */
-          thunar_vfs_info_unref (info);
+          g_object_unref (location);
           g_free (files);
           return NULL;
         }
-      mime_type = thunar_vfs_mime_info_get_name (info->mime_info);
-      files[n].name = thunar_vfs_path_get_name (info->path);
+
+      g_object_unref (location);
+
+      info = thunarx_file_info_get_file_info (lp->data);
+
+      mime_type = g_file_info_get_content_type (info);
+
+      files[n].name = g_file_info_get_name (info);
       files[n].types = types_from_mime_type (mime_type);
-      if (G_UNLIKELY (files[n].types == 0))
-        {
-          mime_database = thunar_vfs_mime_database_get_default ();
-          mime_infos = thunar_vfs_mime_database_get_infos_for_info (mime_database, info->mime_info);
-          for (mp = mime_infos; mp != NULL; mp = mp->next)
-            {
-              mime_type = thunar_vfs_mime_info_get_name (mp->data);
-              files[n].types |= types_from_mime_type (mime_type);
-              thunar_vfs_mime_info_unref (mp->data);
-            }
-          g_object_unref (G_OBJECT (mime_database));
-          g_list_free (mime_infos);
-        }
+
       if (G_UNLIKELY (files[n].types == 0))
         files[n].types = THUNAR_UCA_TYPE_OTHER_FILES;
-      thunar_vfs_info_unref (info);
+
+      g_object_unref (info);
     }
 
   /* lookup the matching items */
@@ -1248,6 +1238,7 @@ thunar_uca_model_update (ThunarUcaModel *uca_model,
                          const gchar    *description,
                          const gchar    *icon,
                          const gchar    *command,
+                         gboolean        startup_notify,
                          const gchar    *patterns,
                          ThunarUcaTypes  types)
 {
@@ -1272,6 +1263,7 @@ thunar_uca_model_update (ThunarUcaModel *uca_model,
   if (G_LIKELY (description != NULL && *description != '\0'))
     item->description = g_strdup (description);
   item->types = types;
+  item->startup_notify = startup_notify;
 
   /* setup the patterns */
   item->patterns = g_strsplit ((patterns != NULL && *patterns != '\0') ? patterns : "*", ";", -1);
@@ -1338,7 +1330,7 @@ thunar_uca_model_save (ThunarUcaModel *uca_model,
   fd = g_mkstemp (tmp_path);
   if (G_UNLIKELY (fd < 0))
     {
-      g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errno), g_strerror (errno));
+      g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errno), "%s", g_strerror (errno));
       goto done;
     }
 
@@ -1367,6 +1359,8 @@ thunar_uca_model_save (ThunarUcaModel *uca_model,
       fprintf (fp, "%s", escaped);
       g_free (patterns);
       g_free (escaped);
+      if (item->startup_notify)
+        fprintf (fp, "<startup-notify/>");
       if ((item->types & THUNAR_UCA_TYPE_DIRECTORIES) != 0)
         fprintf (fp, "<directories/>");
       if ((item->types & THUNAR_UCA_TYPE_AUDIO_FILES) != 0)
@@ -1389,7 +1383,7 @@ thunar_uca_model_save (ThunarUcaModel *uca_model,
   /* rename the file (atomic) */
   if (g_rename (tmp_path, path) < 0)
     {
-      g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errno), g_strerror (errno));
+      g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errno), "%s", g_strerror (errno));
       g_unlink (tmp_path);
     }
   else
