@@ -1,4 +1,4 @@
-/* $Id: thunar-details-view.c 26421 2007-12-02 13:46:28Z benny $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
  *
@@ -41,8 +41,6 @@ enum
 
 
 
-static void         thunar_details_view_class_init              (ThunarDetailsViewClass *klass);
-static void         thunar_details_view_init                    (ThunarDetailsView      *details_view);
 static void         thunar_details_view_finalize                (GObject                *object);
 static void         thunar_details_view_get_property            (GObject                *object,
                                                                  guint                   prop_id,
@@ -131,36 +129,9 @@ static const GtkActionEntry action_entries[] =
   { "setup-columns", NULL, N_ ("Configure _Columns..."), NULL, N_ ("Configure the columns in the detailed list view"), G_CALLBACK (thunar_details_view_action_setup_columns), },
 };
 
-static GObjectClass *thunar_details_view_parent_class;
 
 
-
-GType
-thunar_details_view_get_type (void)
-{
-  static GType type = G_TYPE_INVALID;
-
-  if (G_UNLIKELY (type == G_TYPE_INVALID))
-    {
-      static const GTypeInfo info =
-      {
-        sizeof (ThunarDetailsViewClass),
-        NULL,
-        NULL,
-        (GClassInitFunc) thunar_details_view_class_init,
-        NULL,
-        NULL,
-        sizeof (ThunarDetailsView),
-        0,
-        (GInstanceInitFunc) thunar_details_view_init,
-        NULL,
-      };
-
-      type = g_type_register_static (THUNAR_TYPE_STANDARD_VIEW, I_("ThunarDetailsView"), &info, 0);
-    }
-
-  return type;
-}
+G_DEFINE_TYPE (ThunarDetailsView, thunar_details_view, THUNAR_TYPE_STANDARD_VIEW)
 
 
 
@@ -171,9 +142,6 @@ thunar_details_view_class_init (ThunarDetailsViewClass *klass)
   GtkWidgetClass          *gtkwidget_class;
   GObjectClass            *gobject_class;
 
-  /* determine the parent type class */
-  thunar_details_view_parent_class = g_type_class_peek_parent (klass);
-
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = thunar_details_view_finalize;
   gobject_class->get_property = thunar_details_view_get_property;
@@ -250,9 +218,7 @@ thunar_details_view_init (ThunarDetailsView *details_view)
   gtk_tree_view_set_enable_search (GTK_TREE_VIEW (tree_view), TRUE);
 
   /* enable rubberbanding (if supported) */
-#if GTK_CHECK_VERSION(2,9,0)
   gtk_tree_view_set_rubber_banding (GTK_TREE_VIEW (tree_view), TRUE);
-#endif
 
   /* connect to the default column model */
   details_view->column_model = thunar_column_model_get_default ();
@@ -260,18 +226,18 @@ thunar_details_view_init (ThunarDetailsView *details_view)
 
   /* allocate the shared right-aligned text renderer */
   right_aligned_renderer = g_object_new (THUNAR_TYPE_TEXT_RENDERER, "xalign", 1.0f, NULL);
-  exo_gtk_object_ref_sink (GTK_OBJECT (right_aligned_renderer));
+  g_object_ref_sink (G_OBJECT (right_aligned_renderer));
 
   /* allocate the shared left-aligned text renderer */
   left_aligned_renderer = g_object_new (THUNAR_TYPE_TEXT_RENDERER, "xalign", 0.0f, NULL);
-  exo_gtk_object_ref_sink (GTK_OBJECT (left_aligned_renderer));
+  g_object_ref_sink (G_OBJECT (left_aligned_renderer));
 
   /* allocate the tree view columns */
   for (column = 0; column < THUNAR_N_VISIBLE_COLUMNS; ++column)
     {
       /* allocate the tree view column */
       details_view->columns[column] = gtk_tree_view_column_new ();
-      exo_gtk_object_ref_sink (GTK_OBJECT (details_view->columns[column]));
+      g_object_ref_sink (G_OBJECT (details_view->columns[column]));
       gtk_tree_view_column_set_min_width (details_view->columns[column], 50);
       gtk_tree_view_column_set_resizable (details_view->columns[column], TRUE);
       gtk_tree_view_column_set_sort_column_id (details_view->columns[column], column);
@@ -575,11 +541,7 @@ thunar_details_view_get_visible_range (ThunarStandardView *standard_view,
 {
   _thunar_return_val_if_fail (THUNAR_IS_DETAILS_VIEW (standard_view), FALSE);
 
-#if GTK_CHECK_VERSION(2,8,0)
   return gtk_tree_view_get_visible_range (GTK_TREE_VIEW (GTK_BIN (standard_view)->child), start_path, end_path);
-#else
-  return FALSE;
-#endif
 }
 
 
@@ -809,22 +771,8 @@ thunar_details_view_zoom_level_changed (ThunarDetailsView *details_view)
   /* determine the list of tree view columns */
   for (column = 0; column < THUNAR_N_VISIBLE_COLUMNS; ++column)
     {
-#if GTK_CHECK_VERSION(2,8,0)
       /* just queue a resize on this column */
       gtk_tree_view_column_queue_resize (details_view->columns[column]);
-#else
-      /* determine the renderers for this column */
-      GList *renderers = gtk_tree_view_column_get_cell_renderers (details_view->columns[column]);
-      if (G_LIKELY (renderers != NULL))
-        {
-          /* this is really an awfull hack, but it works! It forces GtkTreeView to recalculate
-           * the dimensions of this column, and that's all that matters. We don't use it with
-           * newer Gtk+ versions either, so what the f*ck...
-           */
-          gtk_tree_view_column_set_cell_data_func (details_view->columns[column], renderers->data, NULL, NULL, NULL);
-          g_list_free (renderers);
-        }
-#endif
     }
 }
 
