@@ -1,4 +1,4 @@
-/* $Id: thunar-private.h 24208 2006-12-28 21:25:19Z benny $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2006 Benedikt Meurer <benny@xfce.org>
  *
@@ -38,23 +38,6 @@ G_BEGIN_DECLS;
 #define _thunar_return_val_if_fail(expr, val) G_STMT_START{ (void)0; }G_STMT_END
 #endif
 
-/* support macros for the slice allocator */
-#if GLIB_CHECK_VERSION(2,10,0)
-#define _thunar_slice_alloc(block_size)             (g_slice_alloc ((block_size)))
-#define _thunar_slice_alloc0(block_size)            (g_slice_alloc0 ((block_size)))
-#define _thunar_slice_free1(block_size, mem_block)  G_STMT_START{ g_slice_free1 ((block_size), (mem_block)); }G_STMT_END
-#define _thunar_slice_new(type)                     (g_slice_new (type))
-#define _thunar_slice_new0(type)                    (g_slice_new0 (type))
-#define _thunar_slice_free(type, ptr)               G_STMT_START{ g_slice_free (type, (ptr)); }G_STMT_END
-#else
-#define _thunar_slice_alloc(block_size)             (g_malloc ((block_size)))
-#define _thunar_slice_alloc0(block_size)            (g_malloc0 ((block_size)))
-#define _thunar_slice_free1(block_size, mem_block)  G_STMT_START{ g_free ((mem_block)); }G_STMT_END
-#define _thunar_slice_new(type)                     (g_new (type, 1))
-#define _thunar_slice_new0(type)                    (g_new0 (type, 1))
-#define _thunar_slice_free(type, ptr)               G_STMT_START{ g_free ((ptr)); }G_STMT_END
-#endif
-
 /* avoid trivial g_value_get_*() function calls */
 #ifdef NDEBUG
 #define g_value_get_boolean(v)  (((const GValue *) (v))->data[0].v_int)
@@ -91,6 +74,23 @@ G_STMT_START{                                                 \
 }G_STMT_END
 #endif
 
+#define XDG_USER_DIRS_PACKAGE "xdg-user-dirs"
+#define LOCALE_FILE_NAME      "user-dirs.locale"
+
+typedef GUserDirectory ThunarUserDirectory;
+#define THUNAR_USER_DIRECTORY_DESKTOP      G_USER_DIRECTORY_DESKTOP
+#define THUNAR_USER_DIRECTORY_DOCUMENTS    G_USER_DIRECTORY_DOCUMENTS
+#define THUNAR_USER_DIRECTORY_DOWNLOAD     G_USER_DIRECTORY_DOWNLOAD
+#define THUNAR_USER_DIRECTORY_MUSIC        G_USER_DIRECTORY_MUSIC
+#define THUNAR_USER_DIRECTORY_PICTURES     G_USER_DIRECTORY_PICTURES
+#define THUNAR_USER_DIRECTORY_PUBLIC_SHARE G_USER_DIRECTORY_PUBLIC_SHARE
+#define THUNAR_USER_DIRECTORY_TEMPLATES    G_USER_DIRECTORY_TEMPLATES
+#define THUNAR_USER_DIRECTORY_VIDEOS       G_USER_DIRECTORY_VIDEOS
+#define THUNAR_USER_N_DIRECTORIES          (8)
+
+gchar *_thunar_get_xdg_user_dirs_locale (void);
+extern const gchar *_thunar_user_directory_names[THUNAR_USER_N_DIRECTORIES+1];
+
 G_END_DECLS;
 
 #endif /* !__THUNAR_PRIVATE_H__ */
