@@ -1,4 +1,4 @@
-/* $Id: thunar-renamer-model.h 20526 2006-03-24 01:50:07Z benny $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
  *
@@ -71,12 +71,26 @@ ThunarxRenamer      *thunar_renamer_model_get_renamer    (ThunarRenamerModel *re
 void                 thunar_renamer_model_set_renamer    (ThunarRenamerModel *renamer_model,
                                                           ThunarxRenamer     *renamer);
 
-void                 thunar_renamer_model_append         (ThunarRenamerModel *renamer_model,
-                                                          ThunarFile         *file);
+void                 thunar_renamer_model_insert         (ThunarRenamerModel *renamer_model,
+                                                          ThunarFile         *file,
+                                                          gint                position);
+void                 thunar_renamer_model_reorder        (ThunarRenamerModel *renamer_model,
+                                                          GList              *tree_paths,
+                                                          gint                position);
 void                 thunar_renamer_model_clear          (ThunarRenamerModel *renamer_model);
 void                 thunar_renamer_model_remove         (ThunarRenamerModel *renamer_model,
                                                           GtkTreePath        *path);
 
+
+/**
+ * thunar_renamer_model_append:
+ * @model : a #ThunarRenamerModel.
+ * @file  : a #ThunarFile instance.
+ *
+ * Appends the @file to the @renamer_model.
+ **/
+#define thunar_renamer_model_append(model,file) thunar_renamer_model_insert (model, file, -1)
+
 G_END_DECLS;
 
 #endif /* !__THUNAR_RENAMER_MODEL_H__ */
